#include "field.h"
#include "expr.h"
#include "keys.h"
#include "io.h"

#include <cmath>

void inputfield::draw(unsigned tick) const
{
	unsigned a;
	char buf[5] = "(00:";
	
	if(base == 2) strncpy(buf, "BIN", 3);
	else if(base == 8) strncpy(buf, "OCT", 3);
	else if(base ==10) strncpy(buf, "DEC", 3);
	else if(base ==16) strncpy(buf, "HEX", 3);
	else if(base ==36) strcpy(buf, "MEGA");
	else 
	{
		buf[1] += base/10;
		buf[2] += base%10;
		buf[3] = ')';
	}
	
	SetChar(x+0, y, buf[0], 0x17,0,0);
	SetChar(x+1, y, buf[1], 0x17,0,0);
	SetChar(x+2, y, buf[2], 0x17,0,0);
	SetChar(x+3, y, buf[3], 0x17,0,0);
	
	for(a=0; a<width; ++a)
	{
		unsigned b = a+alku;
		char c = b < content.size() ? content[b] : ' ';
		if(a==width-1 && b<content.size())
			SetChar(x+a+5, y, '>', selected ? tick<30 && b == cpos ? 0xC4 : 0x03 : 0x09, 0,0);
		else if(!a && alku)
			SetChar(x+a+5, y, '<', selected ? tick<30 && b == cpos ? 0xC4 : 0x03 : 0x09, 0,0);
		else
			SetChar(x+a+5, y, c, selected ? tick<30 && b == cpos ? 0x71 : 0x0B : 0x03, 0,0);
	}
}

void inputfield::locafix()
{
	if(!cpos)alku = 0;
	else if(cpos <= alku)alku = cpos-1;
	else if(cpos > alku && cpos-alku >= width-2)
	{
		if(cpos < content.size())
			alku = cpos-(width-2);
		else
			alku = cpos-(width-1);
	}
}

void inputfield::key(int c)
{
	switch(c)
	{
		case kblt:
			if(cpos > 0)cpos--;
			locafix();
			break;
		case kbrt:
			if(cpos < content.size())cpos++;
			locafix();
			break;
		case kbhome:
			cpos = alku = 0;
			break;
		case kbend:
			alku = 0;
			cpos = content.size();
			locafix();
			break;
		case kbback:
			if(cpos > 0)cpos--;
			/* through */
		case kbdel:
			content.erase(cpos, 1);
			locafix();
			break;
		case '':
			content = "";
			cpos = alku = 0;
			break;
		default:
			c = (unsigned char)c;
			if(c >= ' ')
			{
				content.insert(cpos, 1, (char)c);
				cpos++;
				locafix();
			}
	}
}

void inputfield::laske(string s, unsigned srcbase)
{
	exprp tmp = new expr();
	tmp = tmp->load(s, srcbase, 0);
	
	if(!tmp)
	{
		content = "Expected " + expr_expect;
		if(s.size())
			content += " at `" + s.substr(0, 7) + "'";
		else
			content += ", not eol";
	}
	else if(s.size())
	{
		content = "Parse error at `" + s.substr(0, 7) + "'";
	}
	else
	{
		static const char ss[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		evaltype d = tmp->eval(), expon=0;
		bool nega = d < 0;
		
		if(nega)d = -d;
		
		if(d > powl(base, 8))
		{
			while(d >= powl(base, ++expon));	
			d /= powl(base, --expon);
		}
		
		if(d > 0 && d < powl(base, -8))
		{
			while(d < powl(base, --expon));
			d *= powl(base, ++expon);
		}
		
		unsigned i=0;
		longtype num = (longtype)d;
		d -= num;
		content = "";
		for(; i<width-14; ++i)
		{
			content = ss[num%base] + content;
			num /= base;
			if(!num)break;
		}
		if(nega) content = "-" + content;
		if(d > 0.0 && d < 1.0)
		{
			content += '.';
			for(; i<width-10; ++i)
			{
				d *= base;
				num = (longtype)d;
				content += ss[num];
				d -= num;
				if(d == 0.0)break;
			}
			while(content[content.size()-1]=='0')content.erase(content.size()-1,1);
		}
		
		if(expon != 0)
		{
			string tmp;
			if(content == "-1")
				content = "-10^";
			else if(content == "1")
				content = "10^";
			else
				content += " * 10^";
			if(expon < 0)
			{
				content += "(-";
				expon = -expon;
				tmp = ")";
			}
			while(expon >= 1)
			{
				tmp = ss[(int)fmod(expon, base)] + tmp;
				expon /= base;
			}
			content += tmp;
		}
	}
}
